//
// FullcalendarTimeline - I18n (Internationalisation and locales)
//

// Initialise i18n controls/data
FullcalendarTimeline.prototype.initialiseI18nControls = function () {
    this._initialLocaleCode = navigator.language.toLowerCase(); // e.g. 'en-gb' (navigator.languages might be better)
    this._initialCountryCode = this._initialLocaleCode.substring(0, 2); // e.g. 'en'
    this._initialTimezone = Intl.DateTimeFormat().resolvedOptions().timeZone; // e.g. 'Europe/London'

    // Initialise the locale selector, etc.
    // - Timeline manipulation controls are in fullcalendar_timeline_timeline_manipulation.js
    // - form fields & the highlighter-bar are in FullcalendarTimelineHighlighterBarManipulator
    // - resource arbitration is in FullcalendarTimelineResourceManipulator
    this.initialiseLocaleSelector();
    this.setLocaleCode(this._initialLocaleCode);
}

// Iniialise associated timeline view control elements
FullcalendarTimeline.prototype.initialiseLocaleSelector = function () {
    // Mechanism to pass (this) to the handlers
    var fullCalendarTimelineObject = this;

    var localeSelector = this._timelineViewWrapper.find('#locale-selector');
    var initialLocaleCode = this._initialLocaleCode;
    var initialCountryCode = this._initialCountryCode;

    // Initialise drop-down list of the locale selector
    $.each(FullCalendarLocalesAll, function (index) {
        localeSelector.append(
            $('<option/>')
                .val(FullCalendarLocalesAll[index].code.toLowerCase())
                .text(FullCalendarLocalesAll[index].code)
                //.prop('selected', FullCalendarLocalesAll[index].code == initialLocaleCode)
        );
    });

    // Select a suitable default value
    if (localeSelector.find("option[value='" + initialLocaleCode + "']").length > 0) {
        localeSelector.find("option[value='" + initialLocaleCode + "']").prop('selected', true);
    } else if (localeSelector.find("option[value='" + initialCountryCode + "']").length > 0) {
        localeSelector.find("option[value='" + initialCountryCode + "']").prop('selected', true);
    } else {
        //localeSelector.find("option[value='en-gb']").prop('selected', true);
    }

    // Add event handler to the locale selector
    localeSelector.on('change', function () {
        if (this.value) {
            fullCalendarTimelineObject.setLocaleCode(this.value);
            fullCalendarTimelineObject.renderTopScrollBar();
        }
    });
}

// Utility methods
FullcalendarTimeline.prototype.setLocaleCode = function (localeCode) {
    // TBD: Should only the datetimepickers for the associated form/timeline view
    $.datetimepicker.setLocale(localeCode);
    this._fullcalendarObject.setOption('locale', localeCode);
}
