module ResourcePlanner
  module EventsHelper

    def month_link(month_date)
      link_to(I18n.localize(month_date, :format => "%B"), {:month => month_date.month, :year => month_date.year})
    end

    # custom options for this calendar
    def event_calendar_opts
      {
        :year => @year,
        :month => @month,
        :event_strips => @event_strips,
        :month_name_text => I18n.localize(@shown_month, :format => "%B %Y"),
        :previous_month_text => "<< " + month_link(@shown_month.prev_month),
        :next_month_text => month_link(@shown_month.next_month) + " >>",
        :use_all_day => true
      }
    end

    def event_calendar
      # args is an argument hash containing :event, :day, and :options
      calendar event_calendar_opts do |args|
        event, day = args[:event], args[:day]
        #display_event_details(event, day)
        html = %(<a href="#{mounted_path}/events/#{event.id}" title="#{h(event.name)}">)
        html << display_event_time(event, day)
        html << %(#{h(event.name)}</a>)
        html
      end
    end

    # Multi-line version (doesn't work so well because the <div> height is fixed)
    def display_event_details(event, day)
      html = %(<a href="#{mounted_path}/events/#{event.id}" title="#{h(event.name)}">)

      start_at = event.start_at
      end_at = event.end_at

      html << if !event.all_day and start_at.to_date == day

        start_time = start_at.strftime( (start_at.min == 0) ? "%l" : "%l:%M" )
        start_time += (start_at.strftime("%p") == "PM") ? "pm" : "am"

        end_time = end_at.strftime( (end_at.min == 0) ? "%l" : "%l:%M" )
        end_time += (end_at.strftime("%p") == "PM") ? "pm" : "am"

        %(<span class="ec-event-time">#{start_time}</span> #{h(event.name)}<br/><span class="ec-event-time">#{end_time}</span></a>)
      else
        %(#{h(event.name)}</a>)
      end

      html
    end

    # html for displaying an event's time
    # overrides the default calendar helper
    def display_event_time(event, day)
      start_at = event.start_at
      end_at = event.end_at

      if !event.all_day and start_at.to_date == day

        start_time = start_at.strftime( (start_at.min == 0) ? "%l" : "%l:%M" )
        start_time += (start_at.strftime("%p") == "PM") ? "p" : "a"

        end_time = end_at.strftime( (end_at.min == 0) ? "%l" : "%l:%M" )
        end_time += (end_at.strftime("%p") == "PM") ? "p" : "a"

        %(<span class="ec-event-time">#{start_time}-#{end_time}</span>)
      else
        ""
      end
    end

  end
end
