# frozen_string_literal: true

# Base informational unit for timeline interface
module ResourcePlanner
  class EventUnit < ApplicationRecord

    belongs_to :event, :class_name => '::ResourcePlanner::Event'

    has_many :subscriptions, class_name: '::ResourcePlanner::EventUnitSubscription', inverse_of: :event_unit, dependent: :destroy
    accepts_nested_attributes_for :subscriptions, allow_destroy: true

    # Cannot have a has_many :through association 'ResourcePlanner::EventUnit#resources' on
    # the polymorphic object 'Resource#resource' without 'source_type'. Try adding
    # 'source_type: "Resource"' to 'has_many :through' definition.
    # has_many :resources, :through => :subscriptions, :source => :resource
    # (see resources method, below)

    #--------------------------------------------------------------------------

    #---------- Validations

    #---------- Scopes

    # scope :events, -> { ::ResourcePlanner::Event.events.collect(&:event_units).flatten.uniq }
    # scope :resource_availabilities, -> { ::ResourcePlanner::Event.resource_availabilities.collect(&:event_units).flatten.uniq }
    # scope :resource_restrictions, -> { ::ResourcePlanner::Event.resource_restrictions.collect(&:event_units).flatten.uniq }

    #---------- Callbacks

    #--------------------------------------------------------------------------

    def resources
      self.subscriptions.collect { |a| a.resource }
    end

    #--------------------------------------------------------------------------

    def to_s
      "#{event} - #{start_at}-#{end_at}"
    end

    def all_day?
      all_day
    end
  end
end
